import sys
import bpy
import bmesh
import mathutils

bl_info = {
    "name": "SliceIt",
    "author": "Anthony Aragues, Chipp Walters",
    "version": (1, 0, 5),
    "blender": (3,5,1),
    "location": "View3D > Tool Shelf > Tool",
    "description": "fast, simple slice for selected objects",
    "warning": "",
    "doc_url": "",
    "category": "Mesh",
}

def getBMesh(context, obj, objData=None):
    if obj:
        strMode = context.object.mode
        if strMode == 'EDIT':
            bpy.ops.object.mode_set(mode='OBJECT')
        if objData == None:
            objData = obj.data
        bm = bmesh.new()
        bm.from_mesh(objData)
        if hasattr(bm, 'verts'):
            bm.verts.ensure_lookup_table()
            bm.faces.ensure_lookup_table()
            bm.edges.ensure_lookup_table()
        if strMode == 'OBJECT':
            bpy.ops.object.mode_set(mode='EDIT')
        return bm
    
def setBMesh(context, obj, bm):
    strMode = context.object.mode
    if strMode == 'EDIT':
        bpy.ops.object.mode_set(mode='OBJECT')
    bm.to_mesh(obj.data)
    bm.free()
    if strMode == 'OBJECT':
        bpy.ops.object.mode_set(mode='EDIT')
    return

def getMinMax(obj, mode="WORLD", normal=None):
    # mode == WORLD | LOCAL
    arrVerts = [mathutils.Vector(b) for b in obj.bound_box]
    if mode == "WORLD":
        arrVerts = [obj.matrix_world @ b for b in arrVerts]
    if normal:
        align_to_normal = normal.rotation_difference(
            mathutils.Vector((0.0, 0.0, 1.0))).to_matrix()
        arrVerts = [align_to_normal @ v for v in arrVerts]
    center = sum(arrVerts, mathutils.Vector()) / 8
    minX = min(v[0] for v in arrVerts)
    minY = min(v[1] for v in arrVerts)
    maxX = max(v[0] for v in arrVerts)
    maxY = max(v[1] for v in arrVerts)
    minZ = min(v[2] for v in arrVerts)
    maxZ = max(v[2] for v in arrVerts)
    return {
        "x": {"min": minX, "max": maxX, "center": center[0]},
        "y": {"min": minY, "max": maxY, "center": center[1]},
        "z": {"min": minZ, "max": maxZ, "center": center[2]},
        "origin": obj.matrix_world.to_translation()
    }

class SLICE_OT_Slice(bpy.types.Operator):
    # slice the object with bisecting planes 
    # https://docs.blender.org/api/current/bmesh.ops.html#bmesh.ops.bisect_plane
    # [location, normal]
    bl_idname = 'slice.go'
    bl_label = 'SliceIt'
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        obj = bpy.context.active_object
        bm = getBMesh(context,obj)
        minMax = getMinMax(obj, mode="WORLD")
        min = None
        max = None
        intRange = None
        space = None
        first = mathutils.Vector((minMax['x']['min'],minMax['y']['min'],minMax['z']['min']))
        arrPlanes = []
        if context.scene.slice_axis == 'X':
            normal = (1,0,0)
            min = minMax['x']['min']
            max = minMax['x']['max']
            intAxis = 0
        elif context.scene.slice_axis == 'Y':
            normal = (0,1,0)
            min = minMax['y']['min']
            max = minMax['y']['max']
            intAxis = 1
        elif context.scene.slice_axis == 'Z':
            normal = (0,0,1)
            min = minMax['z']['min']
            max = minMax['z']['max']
            intAxis = 2
        intRange = max - min
        space = intRange / context.scene.slice_count
        for i in range(int(context.scene.slice_count)):
            arrVector = first.copy()
            arrVector[intAxis] = arrVector[intAxis] + (space * i)
            arrPlanes.append([mathutils.Vector(arrVector), normal])
        for plane in arrPlanes:
            arrEdges = [e for e in bm.edges if e.select]
            if len(arrEdges) == 0:
                arrEdges = [e for e in bm.edges]
            geo = list(bm.verts) + arrEdges + list(bm.faces)
            bmesh.ops.bisect_plane(
                bm,
                geom=geo,
                dist=0,
                plane_co=plane[0],
                plane_no=plane[1],
                clear_inner=False,
                clear_outer=False)
        bmesh.ops.remove_doubles(bm, verts=bm.verts, dist=0.0001)
        obj.data.update()
        setBMesh(context,obj, bm)
        return {'FINISHED'}

class SLICE_PT_Main(bpy.types.Panel):
    bl_label = "SliceIt"
    bl_category = "Tool"
    bl_idname = "SLICE_PT_Main"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    #create the panel
    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row = row.label(text="Slice Direction")
        row = layout.row()
        row.prop(context.scene, "slice_axis", expand=True)
        row = layout.row()
        row.prop(context.scene, "slice_count")
        row = layout.row()
        row.operator("slice.go")

arrClasses = [
    SLICE_PT_Main,
    SLICE_OT_Slice
]

def cleanse_modules():
    for module_name in sorted(sys.modules.keys()):
        if module_name.startswith(__name__):
            del sys.modules[module_name]

def register():
    for i in arrClasses:
        bpy.utils.register_class(i)
    bpy.types.Scene.slice_count = bpy.props.IntProperty(name="Num Slices", default=10, min=1, description="Number of slices to make")
    bpy.types.Scene.slice_axis = bpy.props.EnumProperty( name="Slice Direction", items=[
        ('X','X','evenly spaced slices on x axis'),
        ('Y','Y','evenly spaced slices on y axis'),
        ('Z','Z','evenly spaced slices on z axis')
    ], default='X' )

def unregister():
    for i in arrClasses:
        bpy.utils.unregister_class(i)
    del bpy.types.Scene.slice_count
    del bpy.types.Scene.slice_axis
    cleanse_modules()
    